import * as tslib_1 from "tslib";
import { compose, mapProps, setDisplayName, withState, } from 'recompose';
/**
 * Given an error from yup validation, turn it into form errors
 */
export function yupToFormErrors(yupError) {
    var errors = {};
    for (var _i = 0, _a = yupError.inner; _i < _a.length; _i++) {
        var err = _a[_i];
        if (!errors[err.path]) {
            errors[err.path] = err.message;
        }
    }
    return errors;
}
/**
 * Given a FormState, return a `touched` value with all of the fields touched.
 */
export function touchAllFields(fields) {
    var touched = {};
    for (var _i = 0, _a = Object.keys(fields); _i < _a.length; _i++) {
        var k = _a[_i];
        touched[k] = true;
    }
    return touched;
}
export function validateFormData(data, schema, setErrors) {
    var validateData = {};
    for (var k in data) {
        if (data.hasOwnProperty(k)) {
            var key = String(k);
            validateData[key] = data[key] !== ''
                ? data[key]
                : undefined;
        }
    }
    return schema.validate(validateData, { abortEarly: false }).then(function () {
        setErrors({});
        return true;
    }, function (err) {
        setErrors(yupToFormErrors(err));
        return false;
    });
}
export default function Formik(_a) {
    var displayName = _a.displayName, _b = _a.mapPropsToValues, mapPropsToValues = _b === void 0 ? function (formProps) { return formProps; } : _b, _c = _a.mapValuesToPayload, mapValuesToPayload = _c === void 0 ? function (formPartialState) {
        // in this case State and Payload are the same.
        var payload = formPartialState;
        return payload;
    } : _c, validationSchema = _a.validationSchema, handleSubmit = _a.handleSubmit;
    return compose(setDisplayName(displayName), withState('values', 'setValues', function (props) { return mapPropsToValues(props); }), withState('errors', 'setErrors', {}), withState('error', 'setError', undefined), withState('touched', 'setTouched', {}), withState('isSubmitting', 'setSubmitting', false), mapProps(function (_a) {
        var values = _a.values, error = _a.error, errors = _a.errors, touched = _a.touched, isSubmitting = _a.isSubmitting, setError = _a.setError, setErrors = _a.setErrors, setValues = _a.setValues, setTouched = _a.setTouched, setSubmitting = _a.setSubmitting, rest = tslib_1.__rest(_a, ["values", "error", "errors", "touched", "isSubmitting", "setError", "setErrors", "setValues", "setTouched", "setSubmitting"]);
        return (tslib_1.__assign({ onChange: function (e) {
                e.persist();
                var _a = e.target, type = _a.type, name = _a.name, value = _a.value, checked = _a.checked;
                var val = /number|range/.test(type)
                    ? parseFloat(value)
                    : /checkbox/.test(type)
                        ? checked
                        : /radio/.test(type) // is this needed?
                            ? value
                            : value;
                // Set form fields by name
                setValues(tslib_1.__assign({}, values, (_b = {}, _b[name] = val, _b)));
                // Validate against schema
                validateFormData(tslib_1.__assign({}, values, (_c = {}, _c[name] = val, _c)), validationSchema, setErrors);
                var _b, _c;
            }, onBlur: function (e) {
                e.persist();
                setTouched(tslib_1.__assign({}, values, (_a = {}, _a[e.target.name] = true, _a)));
                var _a;
            }, onChangeValue: function (name, value) {
                // Set changed fields as touched
                setTouched(tslib_1.__assign({}, touched, (_a = {}, _a[name] = true, _a)));
                // Set form fields by name
                setValues(tslib_1.__assign({}, values, (_b = {}, _b[name] = value, _b)));
                // Validate against schema
                validateFormData(tslib_1.__assign({}, values, (_c = {}, _c[name] = value, _c)), validationSchema, setErrors);
                var _a, _b, _c;
            }, onSubmit: function (e) {
                e.preventDefault();
                setTouched(touchAllFields(values));
                setSubmitting(true);
                setErrors({});
                setError(undefined);
                validateFormData(values, validationSchema, setErrors).then(function (isValid) {
                    console.log('isValid:' + isValid);
                    if (isValid) {
                        handleSubmit(mapValuesToPayload(values), {
                            setTouched: setTouched,
                            setErrors: setErrors,
                            setError: setError,
                            setSubmitting: setSubmitting,
                            setValues: setValues,
                            props: rest,
                        });
                    }
                });
            }, resetForm: function (nextProps) {
                setSubmitting(false);
                setErrors({});
                setTouched({});
                setError(undefined);
                if (nextProps) {
                    setValues(mapPropsToValues(nextProps));
                }
                else {
                    setValues(mapPropsToValues(rest));
                }
            }, onReset: function () {
                setSubmitting(false);
                setErrors({});
                setTouched({});
                setError(undefined);
                setValues(mapPropsToValues(rest));
            }, setValues: setValues,
            setErrors: setErrors,
            setSubmitting: setSubmitting,
            errors: errors,
            error: error,
            isSubmitting: isSubmitting,
            touched: touched,
            values: values }, rest));
    }));
}
